/*
    ettercap -- data decoding module

    Copyright (C) 2001  ALoR <alor@thepentagon.com>, NaGA <crwm@freemail.it>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/


#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>

#include "ec_main.h"

#ifdef DEBUG
	#include "ec_debug.h"
#endif

typedef struct
{
	char ip_source[16];
	char ip_dest[16];
	int port_source;
	int port_dest;
	char status;
	struct conn_list *next;
} conn_list;


conn_list *c_list = NULL;


int Decodedata_MakeConnectionList(CONNECTION data);
int Decodedata_RefreshConnectionList(void);
void Decodedata_Free_list(conn_list	*head);
void Decodedata_Make_list(CONNECTION data);
char * Decodedata_GetAsciiData(char *buffer, int buff_len);
char * Decodedata_GetHexData(char *buffer, int buff_len, short dimX);
char * Decodedata_GetEnhanchedHexData(char *buffer, int buff_len, short cr);

//--------------------------


int Decodedata_MakeConnectionList(CONNECTION data)
{
	int num_conn = 0, i = 0;
	conn_list *c_index;


	Decodedata_Make_list(data);

	c_index = c_list;

	for( ; c_index; num_conn++)
		c_index = (conn_list *)c_index->next;

	if	(Conn_Between_Hosts) free(Conn_Between_Hosts);
	Conn_Between_Hosts = (CONNECTION *)calloc(num_conn, sizeof(CONNECTION));

	c_index = c_list;

	for( ; c_index; c_index = (conn_list *)c_index->next)
	{
		memcpy(&Conn_Between_Hosts[i].ip_source, &c_index->ip_source, sizeof(c_index->ip_source));
		memcpy(&Conn_Between_Hosts[i].ip_dest, &c_index->ip_dest, sizeof(c_index->ip_dest));
		memcpy(&Conn_Between_Hosts[i].port_source, &c_index->port_source, sizeof(int));
		memcpy(&Conn_Between_Hosts[i].port_dest, &c_index->port_dest, sizeof(int));
		memcpy(&Conn_Between_Hosts[i].status, &c_index->status, sizeof(char));
		sprintf(Conn_Between_Hosts[i].type,	"..TODO..");			// TODO
		i++;
	}

	return num_conn;

}


int Decodedata_RefreshConnectionList(void)
{

#ifdef DEBUG
	Debug_msg("Decodedata_RefreshConnectionList");
#endif

	if	(Conn_Between_Hosts) free(Conn_Between_Hosts);
	Conn_Between_Hosts = NULL;
	Decodedata_Free_list(c_list);
	c_list = NULL;

	return 0;
}



void Decodedata_Free_list(conn_list	*head)
{
	if (!head)	return;

	Decodedata_Free_list((conn_list *) head->next);
	free(head);
}



void Decodedata_Make_list(CONNECTION data)
{
	conn_list *node;
	conn_list *c_index = NULL;
	conn_list *temp;
	char filter[45];
	char found = 0;

	sprintf(filter, "%s%d%s%d", data.ip_source, data.port_source, data.ip_dest, data.port_dest);

	temp = c_list;

	while (temp != NULL)
	{
		char cur_conn[45];
		char rev_cur_conn[45];
		sprintf(cur_conn, "%s%d%s%d", temp->ip_source, temp->port_source, temp->ip_dest, temp->port_dest);
		sprintf(rev_cur_conn, "%s%d%s%d", temp->ip_dest, temp->port_dest, temp->ip_source, temp->port_source);

		temp->status = 0;	// silent

		if (!strcmp(filter, cur_conn)) { temp->status = 1; found = 1; }	// ACTIVE
		if (!strcmp(filter, rev_cur_conn)) { temp->status = 1; found = 1; }

		if (temp != NULL) c_index = temp;

		temp = (conn_list *)temp->next;
	}

	if (!found)
	{
		#ifdef DEBUG
			Debug_msg("Decodedata_Make_list - new node ! %s:%d - %s:%d", data.ip_source, data.port_source, data.ip_dest, data.port_dest);
		#endif

		node = (conn_list *)malloc(sizeof(conn_list));
		node->next=NULL;

		memcpy(&node->ip_source, &data.ip_source, sizeof(data.ip_source));
		memcpy(&node->ip_dest, &data.ip_dest, sizeof(data.ip_dest));
		memcpy(&node->port_source, &data.port_source, sizeof(int));
		memcpy(&node->port_dest, &data.port_dest, sizeof(int));
		node->status = 1;

		if (c_index != NULL)
			(conn_list *)c_index->next = node;

		if (c_list == NULL) c_list = node;
	}

}



char * Decodedata_GetHexData(char *buffer, int buff_len, short dimX)
{
	short octets;

	for(octets = 0; octets < dimX; octets++)
	{
		if ( (octets*3.5 + 12) >= dimX ) break;
	}

	if (octets > 16) octets = 16;
	if (octets % 2 == 1) octets--;

	return Decodedata_GetEnhanchedHexData(buffer, buff_len, octets);

}


char * Decodedata_GetEnhanchedHexData(char *buffer, int buff_len, short cr)
{
	static char *hexdata;
   int i, j, jm;
   int c, dim = 0;

	if (buff_len == 0) return "";

	c = cr*3.5 + 11;
	dim = c;

	for (i = 0; i < buff_len; i++)	// approximately
		if ( i % cr == 0)					// approximately
			dim += c;						// approximately


	if (hexdata) free(hexdata);
	hexdata = (char *)calloc(dim, sizeof(char));

	// adapted from dsniff by Dug Song <dugsong@monkey.org>
   sprintf(hexdata,"\n");
   for (i = 0; i < buff_len; i += cr) {
           sprintf(hexdata, "%s %04x: ", hexdata, i );
           jm = buff_len - i;
           jm = jm > cr ? cr : jm;

           for (j = 0; j < jm; j++) {
                   if ((j % 2) == 1) sprintf(hexdata,"%s%02x ", hexdata, (unsigned char) buffer[i+j]);
                   else sprintf(hexdata,"%s%02x", hexdata, (unsigned char) buffer[i+j]);
           }
           for (; j < cr; j++) {
                   if ((j % 2) == 1) strcat(hexdata,"   ");
                   else strcat(hexdata,"  ");
           }
           strcat(hexdata," ");

           for (j = 0; j < jm; j++) {
                   c = buffer[i+j];
                   c = isprint(c) ? c : '.';
                   sprintf(hexdata,"%s%c", hexdata, c);
           }
           strcat(hexdata,"\n");
   }

	return hexdata;
}



char * Decodedata_GetAsciiData(char *buffer, int buff_len)
{

	int i = 0;

	if (buff_len == 0) return "";

	for(i = 0; i < buff_len; i++)
	{
		if ( !( isprint((int)buffer[i]) || buffer[i] == '\n' || buffer[i] == '\t') )
			buffer[i] = '.';
	}

	return buffer;
}

