/*
    ettercap -- illithid -- the sniffer module

    Copyright (C) 2001  ALoR <alor@thepentagon.com>, NaGA <crwm@freemail.it>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#include <stdio.h>
#include <sys/time.h>
#include <signal.h>
#include <netpacket/packet.h>
#include <net/if_arp.h>
#include <net/ethernet.h>
#include <sys/ioctl.h>
#include <net/if.h>
#include <netinet/in.h>
#include <netdb.h>
#include <netinet/tcp.h>
#include <unistd.h>
#include <sys/socket.h>
#include <arpa/inet.h>
#include <string.h>
#include <fcntl.h>
#include <errno.h>

#include "ec_main.h"
#include "ec_inet.h"

#ifdef DEBUG
	#include "ec_debug.h"
#endif

#define ARPBased 0
#define IPBased  1
#define MACBased 2


struct ETH_packet
{
	char DestMAC[6];
	char SourceMAC[6];
	unsigned short Protocol_type;
};

typedef struct iphdr {
	unsigned char h_len:4;
	unsigned char version:4;
	unsigned char tos;
	unsigned short total_len;
	unsigned short ident;
	unsigned short frag_and_flags;
	unsigned char  ttl;
	unsigned char proto;
	unsigned short checksum;
	unsigned int sourceIP;
	unsigned int destIP;
}IpHeader;


// protos...

char *Illithid_packet_forge(char towrite, int IPS, short PortS, char *MACS, int IPD, short PortD, char *MACD, int Seq, int Ack, int Ident);

int Illithid_ARPBased_GetData(char *iface, char *IPsource, int PortSource, char *IPdest, int PortDest, char *MAC1, char *MAC2);
int Illithid_IPBased_GetData(char *iface, char *IPsource, int PortSource, char *IPdest, int PortDest);
int Illithid_MACBased_GetData(char *iface, char *MACsource, char *MACdest);

int Illithid_ARPBased_GetConnections(char *iface, char *IP1p, char *IP2p, char *MAC1, char *MAC2);
int Illithid_IPBased_GetConnections(char *iface, char *IP1p, char *IP2p);
int Illithid_MACBased_GetConnections(char *iface, char *MAC1p, char *MAC2p);

void Illithid_GetConnections(short mode, char *iface, char *IP1p, char *IP2p, char *MAC1p, char *MAC2p);

int Illithid_ToBeSniffed(char *source, int psource, char *dest, int pdest, SNIFFED_DATA data);

//---------------------------




char *Illithid_packet_forge(char towrite, int IPS, short PortS, char *MACS, int IPD, short PortD, char *MACD, int Seq, int Ack, int Ident)
{
	struct ETH_packet *eth;
	struct tcphdr *tcp;
	IpHeader *ip;
	char * data;
	static char pck_buffer[1024];

#ifdef DEBUG
	Debug_msg("Illithid_packet_forge");
#endif

	memset(pck_buffer,0,1024);

	eth = (struct ETH_packet *)pck_buffer;
	ip = (IpHeader *) (eth + 1);
	tcp = (struct tcphdr *) (ip + 1);
	data = (char *) (tcp + 1);

	memcpy(eth->DestMAC,MACD,6);
	memcpy(eth->SourceMAC, MACS, 6);
	eth->Protocol_type = htons(ETHERTYPE_IP);

	ip->h_len = 5;
	ip->version = 4;
	ip->tos = 0;
	ip->total_len = htons(sizeof(IpHeader)+sizeof(struct tcphdr)+1);
	ip->ident = htons(Ident);
	ip->frag_and_flags = 0;
	ip->ttl = 125;
	ip->proto = IPPROTO_TCP;
	ip->checksum = 0;
	ip->sourceIP = IPS;
	ip->destIP = IPD;
	ip->checksum = Inet_ChecksumIP( (unsigned short *) ip, sizeof(IpHeader));


	tcp->source = htons(PortS);
	tcp->dest = htons(PortD);
	tcp->seq = htonl(Seq);
	tcp->ack_seq = htonl(Ack);
	tcp->ack = 1;
	tcp->psh = 1;
	tcp->doff = sizeof(struct tcphdr) / 4;
	tcp->urg_ptr = 0;
	tcp->check = 0;
	tcp->window = htons(10000);


	*data = towrite;

	tcp->check = Inet_ChecksumTCP((unsigned short *)tcp, ntohs(ip->total_len)-ip->h_len*4 , IPS, IPD);

	return pck_buffer;
}



int Illithid_ARPBased_GetData(char *iface, char *IPsource, int PortSource, char *IPdest, int PortDest, char *MAC1, char *MAC2)
{
	int ifnumber, IP1, IP2, datalen, MTU;
	struct sockaddr_ll from;
	struct ETH_packet *eth;
	struct sockaddr_ll dest;
	int sock;
	IpHeader *ip;
	struct tcphdr *tcp;
	unsigned char PoorMAC1[6];
	unsigned char PoorMAC2[6];
	char MyMAC[6];
	unsigned char *dati;
	struct in_addr addr;
	SNIFFED_DATA data_to_ettercap;
	char tosendchar,nchars;
	char *ins_pck;
	int Illithid_pid;

	short IPIDD=0, IPIDS=0;
	int SeqS=0, AckS=0, SeqD=0, AckD=0, ByteSentS=0, ByteSentD=0;
	int datalenS=0, datalenD=0;

#ifdef DEBUG
	Debug_msg("Illithid_ARPBased_GetData -- %s:%d %s:%d -- %s %s", IPsource, PortSource, IPdest, PortDest, MAC1, MAC2);
#endif

	if ((Illithid_pid = fork()))
		return Illithid_pid;

	strcpy(program_argv0, "illithid");

	inet_aton(IPsource,&addr);
	IP1 = addr.s_addr;
	inet_aton(IPdest,&addr);
	IP2 = addr.s_addr;

	sock = Inet_GetRawSock(iface, &MTU, &ifnumber, MyMAC);

	sscanf(MAC1,"%02X:%02X:%02X:%02X:%02X:%02X",(unsigned int *)&PoorMAC1[0],
		(unsigned int *)&PoorMAC1[1],(unsigned int *)&PoorMAC1[2],
		(unsigned int *)&PoorMAC1[3],(unsigned int *)&PoorMAC1[4],
		(unsigned int *)&PoorMAC1[5]);
	sscanf(MAC2,"%02X:%02X:%02X:%02X:%02X:%02X",(unsigned int *)&PoorMAC2[0],
		(unsigned int *)&PoorMAC2[1],(unsigned int *)&PoorMAC2[2],
		(unsigned int *)&PoorMAC2[3],(unsigned int *)&PoorMAC2[4],
		(unsigned int *)&PoorMAC2[5]);

	dest.sll_family = AF_PACKET;
	dest.sll_ifindex = ifnumber;

	fcntl(sock,F_SETFL,O_NONBLOCK);
	fcntl(pipe_inject_stod[0], F_SETFL, O_NONBLOCK);
	fcntl(pipe_inject_dtos[0], F_SETFL, O_NONBLOCK);


	for (;;)
	{
		int len;
		char buffer[MTU];
		char ok;

		memset(buffer, 0, MTU);

		len = Inet_GetRawPacket(sock, buffer, MTU, &from);

		if (len > 0 && from.sll_pkttype == PACKET_HOST)
		{
			eth = (struct ETH_packet *)buffer;

			if ( ntohs(eth->Protocol_type) == ETHERTYPE_IP )
			{
				ip = (struct iphdr *)(eth+1);
				ok = 0;

				if (ip->destIP==IP1 && ip->sourceIP==IP2)
				{
					memcpy(eth->DestMAC, PoorMAC1, 6);
					memcpy(eth->SourceMAC, MyMAC, 6);
					ok = 1;
				}

				if (ip->destIP==IP2 && ip->sourceIP==IP1)
				{
					memcpy(eth->DestMAC, PoorMAC2, 6);
					memcpy(eth->SourceMAC, MyMAC, 6);
					ok = 2;
				}

				strcpy(data_to_ettercap.ip_source, inet_ntoa(*(struct in_addr *)&ip->sourceIP) );
				strcpy(data_to_ettercap.ip_dest, inet_ntoa(*(struct in_addr *)&ip->destIP) );

				if (ok && ip->proto == IPPROTO_TCP)
				{
					tcp = (struct tcphdr *) ((int)ip + ip->h_len * 4);


					data_to_ettercap.port_source = ntohs(tcp->source);
					data_to_ettercap.port_dest = ntohs(tcp->dest);

					if ( Illithid_ToBeSniffed(IPsource,PortSource, IPdest, PortDest, data_to_ettercap) )
					{
						if (ok == 1)
						{
							SeqS = ntohl(tcp->seq);
							AckS = ntohl(tcp->ack_seq);
							tcp->seq = htonl(SeqS+ByteSentS);
							tcp->ack_seq = htonl(AckS-ByteSentD);
							IPIDS = ntohs(ip->ident);
							ip->ident = htons(IPIDS+ByteSentS);
							dati = (char *)((int)tcp + tcp->doff * 4);
							datalenS = (int)ip + ntohs(ip->total_len) - (int)dati;

							ip->checksum = 0;
							tcp->check = 0;
							ip->checksum = Inet_ChecksumIP( (unsigned short *) ip, sizeof(IpHeader));
							tcp->check = Inet_ChecksumTCP((unsigned short *)tcp, ntohs(ip->total_len)-ip->h_len*4, IP2, IP1);
						}
						else
						{
							SeqD = ntohl(tcp->seq);
							AckD = ntohl(tcp->ack_seq);
							tcp->seq = htonl(SeqD+ByteSentD);
							tcp->ack_seq = htonl(AckD-ByteSentS);
							IPIDD = ntohs(ip->ident);
							ip->ident = htons(IPIDD+ByteSentD);
							dati = (char *)((int)tcp + tcp->doff * 4);
							datalenD = (int)ip + ntohs(ip->total_len) - (int)dati;

							ip->checksum = 0;
							tcp->check = 0;
							ip->checksum = Inet_ChecksumIP( (unsigned short *) ip, sizeof(IpHeader));
							tcp->check = Inet_ChecksumTCP((unsigned short *)tcp, ntohs(ip->total_len)-ip->h_len*4, IP2, IP1);
						}

						dati = (char *)((int)tcp + tcp->doff * 4);
						datalen = (int)ip + ntohs(ip->total_len)  - (int)dati;

						data_to_ettercap.datasize = datalen;
						memset(&data_to_ettercap.data, 0, sizeof(data_to_ettercap.data));
						memcpy(&data_to_ettercap.data, dati, datalen);

						write(pipe_with_illithid_data[1], &data_to_ettercap, sizeof(SNIFFED_DATA));

					}
				}
				if (ok) Inet_SendRawPacket(sock, buffer, len, dest);
			}
		}


		nchars = read(pipe_inject_stod[0],&tosendchar,1);
		if (nchars==1 && tosendchar)
		{
			ins_pck=(char *)Illithid_packet_forge(tosendchar, IP1, PortSource, PoorMAC1, IP2, PortDest, PoorMAC2, SeqD+datalenD+ByteSentD, AckD-ByteSentS, IPIDD+ByteSentD+1);
			sendto(sock, ins_pck, 60, 0, (struct sockaddr *)&dest, sizeof(dest));
			ByteSentD++;
		}

		nchars = read(pipe_inject_dtos[0],&tosendchar,1);
		if (nchars==1 && tosendchar)
		{
			ins_pck=(char *)Illithid_packet_forge(tosendchar, IP2, PortDest, PoorMAC2, IP1, PortSource, PoorMAC1, SeqS+datalenS+ByteSentS, AckS-ByteSentD, IPIDS+ByteSentS+1);
			sendto(sock, ins_pck, 60, 0, (struct sockaddr *)&dest, sizeof(dest));
			ByteSentS++;
		}

	}

}







int Illithid_IPBased_GetData(char *iface, char *IPsource, int PortSource, char *IPdest, int PortDest)
{

	int IP1, IP2, datalen, len, MTU, sock;
	struct sockaddr_ll from;
	struct ETH_packet *eth;
	IpHeader *ip;
	struct tcphdr *tcp;
	unsigned char *dati;
	struct in_addr addr;
	SNIFFED_DATA data_to_ettercap;
	fd_set msk_sock;
	struct timeval TimeOut;
	int Illithid_pid;

#ifdef DEBUG
	Debug_msg("Illithid_IPBased_GetData -- %s:%d %s:%d", IPsource, PortSource, IPdest, PortDest);
#endif

	if ((Illithid_pid = fork()))
		return Illithid_pid;

	strcpy(program_argv0, "illithid");


	inet_aton(IPsource,&addr);
	IP1 = addr.s_addr;
	inet_aton(IPdest,&addr);
	IP2 = addr.s_addr;

	sock = Inet_GetRawSock(iface, &MTU, NULL, NULL);

	memset(&TimeOut, 0, sizeof(TimeOut));	//  timeout = 0
	FD_ZERO(&msk_sock);

	for (;;)
	{
		char buffer[MTU];

		FD_SET(sock, &msk_sock);
 		select(FOPEN_MAX, &msk_sock, (fd_set *) 0, (fd_set *) 0, &TimeOut);

		if (FD_ISSET(sock, &msk_sock))
		{

			memset(&data_to_ettercap, 0, sizeof(SNIFFED_DATA));

			len = Inet_GetRawPacket(sock, buffer, MTU, &from);

			if (len > 0)
			{
				eth = (struct ETH_packet *)buffer;

				if ( ntohs(eth->Protocol_type) == ETHERTYPE_IP )
				{
					ip = (struct iphdr *)(eth+1);

					strcpy(data_to_ettercap.ip_source, inet_ntoa(*(struct in_addr *)&ip->sourceIP) );
					strcpy(data_to_ettercap.ip_dest, inet_ntoa(*(struct in_addr *)&ip->destIP) );

					if (ip->proto == IPPROTO_TCP)
					{
						tcp = (struct tcphdr *) (ip + 1);

						data_to_ettercap.port_source = ntohs(tcp->source);
						data_to_ettercap.port_dest = ntohs(tcp->dest);

						if ( Illithid_ToBeSniffed(IPsource,PortSource, IPdest, PortDest, data_to_ettercap) )
						{
							dati =  (char *)((int)tcp + tcp->doff * 4);
							datalen = (int)ip + ntohs(ip->total_len)  - (int)dati;

							data_to_ettercap.datasize = datalen;
							memcpy(&data_to_ettercap.data, dati, datalen);

							write(pipe_with_illithid_data[1], &data_to_ettercap, sizeof(SNIFFED_DATA));

						}
					}
				}
			}
		}
		else
			usleep(1);
	}


}



int Illithid_MACBased_GetData(char *iface, char *MAC1p, char *MAC2p)
{

	int datalen, len, MTU, sock;
	unsigned char MAC1[6];
	unsigned char MAC2[6];
	struct sockaddr_ll from;
	struct ETH_packet *eth;
	IpHeader *ip;
	struct tcphdr *tcp;
	unsigned char *dati;
	SNIFFED_DATA data_to_ettercap;
	fd_set msk_sock;
	struct timeval TimeOut;
	int Illithid_pid;

#ifdef DEBUG
	Debug_msg("Illithid_MACBased_GetData -- %s %s", MAC1p, MAC2p);
#endif

	if ((Illithid_pid = fork()))
		return Illithid_pid;


	strcpy(program_argv0, "illithid");


	sscanf(MAC1p,"%02X:%02X:%02X:%02X:%02X:%02X",(unsigned int *)&MAC1[0],
		(unsigned int *)&MAC1[1],(unsigned int *)&MAC1[2],
		(unsigned int *)&MAC1[3],(unsigned int *)&MAC1[4],
		(unsigned int *)&MAC1[5]);
	sscanf(MAC2p,"%02X:%02X:%02X:%02X:%02X:%02X",(unsigned int *)&MAC2[0],
		(unsigned int *)&MAC2[1],(unsigned int *)&MAC2[2],
		(unsigned int *)&MAC2[3],(unsigned int *)&MAC2[4],
		(unsigned int *)&MAC2[5]);

	sock = Inet_GetRawSock(iface, &MTU, NULL, NULL);

	memset(&TimeOut, 0, sizeof(TimeOut));	//  timeout = 0
	FD_ZERO(&msk_sock);

	for (;;)
	{
		char buffer[MTU];

		FD_SET(sock, &msk_sock);
		select(FOPEN_MAX, &msk_sock, (fd_set *) 0, (fd_set *) 0, &TimeOut);

		if (FD_ISSET(sock, &msk_sock))
		{

			memset(&data_to_ettercap, 0, sizeof(SNIFFED_DATA));

			len = Inet_GetRawPacket(sock, buffer, MTU, &from);

			if (len > 0)
			{
				char ok = 0;

				eth = (struct ETH_packet *)buffer;

				if ( memcmp(eth->SourceMAC, MAC1, 6) == 0 || memcmp(eth->SourceMAC, MAC2, 6) == 0 ) ok = 1;
				if ( memcmp(eth->DestMAC, MAC1, 6) == 0 || memcmp(eth->DestMAC, MAC2, 6) == 0 ) ok = 1;

				if ( ok && ntohs(eth->Protocol_type) == ETHERTYPE_IP )
				{
					ip = (struct iphdr *)(eth+1);

					strcpy(data_to_ettercap.ip_source, inet_ntoa(*(struct in_addr *)&ip->sourceIP) );
					strcpy(data_to_ettercap.ip_dest, inet_ntoa(*(struct in_addr *)&ip->destIP) );

					if ( ip->proto == IPPROTO_TCP)
					{
						tcp = (struct tcphdr *)(ip + 1);

						data_to_ettercap.port_source = ntohs(tcp->source);
						data_to_ettercap.port_dest = ntohs(tcp->dest);

						dati = (char *)((int)tcp + tcp->doff * 4);
						datalen = (int)ip + ntohs(ip->total_len)  - (int)dati;

						data_to_ettercap.datasize = datalen;
						memcpy(&data_to_ettercap.data, dati, datalen);

						write(pipe_with_illithid_data[1], &data_to_ettercap, sizeof(SNIFFED_DATA));
					}
				}
			}
		}
		else
			usleep(1);
	}


}



int Illithid_ARPBased_GetConnections(char *iface, char *IP1p, char *IP2p, char *MAC1p, char *MAC2p)
{
	int Illithid_pid;

#ifdef DEBUG
	Debug_msg("Illithid_ARPBased_GetConnections -- %s %s %s %s", IP1p, IP2p, MAC1p, MAC2p);
#endif

	if (!(Illithid_pid = fork()))
		Illithid_GetConnections(ARPBased, iface, IP1p, IP2p, MAC1p, MAC2p);
	else
		return Illithid_pid;

	return 0;
}



int Illithid_IPBased_GetConnections(char *iface, char *IP1p, char *IP2p)
{
		int Illithid_pid;

#ifdef DEBUG
	Debug_msg("Illithid_IPBased_GetConnections -- %s %s", IP1p, IP2p);
#endif

	if (!(Illithid_pid = fork()))
		Illithid_GetConnections(IPBased, iface, IP1p, IP2p, "", "");
	else
		return Illithid_pid;

	return 0;
}


int Illithid_MACBased_GetConnections(char *iface, char *MAC1p, char *MAC2p)
{
		int Illithid_pid;

#ifdef DEBUG
	Debug_msg("Illithid_MACBased_GetConnections -- %s %s", MAC1p, MAC2p);
#endif

	if (!(Illithid_pid = fork()))
		Illithid_GetConnections(MACBased, iface, "", "", MAC1p, MAC2p);
	else
		return Illithid_pid;

	return 0;
}



void Illithid_GetConnections(short mode, char *iface, char *IP1p, char *IP2p, char *MAC1p, char *MAC2p)
{
	int len, MTU, IP1, IP2, ifnumber, sock;
	unsigned char MAC1[6];
	unsigned char MAC2[6];
	unsigned char MyMAC[6];
	struct sockaddr_ll from;
	struct sockaddr_ll dest;
	struct in_addr addr;
	struct ETH_packet *eth;
	IpHeader *ip;
	struct tcphdr *tcp;
	CONNECTION data_to_ettercap;

	strcpy(program_argv0, "illithid");

	inet_aton(IP1p,&addr);
	IP1 = addr.s_addr;
	inet_aton(IP2p,&addr);
	IP2 = addr.s_addr;

	sscanf(MAC1p,"%02X:%02X:%02X:%02X:%02X:%02X",(unsigned int *)&MAC1[0],
		(unsigned int *)&MAC1[1],(unsigned int *)&MAC1[2],
		(unsigned int *)&MAC1[3],(unsigned int *)&MAC1[4],
		(unsigned int *)&MAC1[5]);
	sscanf(MAC2p,"%02X:%02X:%02X:%02X:%02X:%02X",(unsigned int *)&MAC2[0],
		(unsigned int *)&MAC2[1],(unsigned int *)&MAC2[2],
		(unsigned int *)&MAC2[3],(unsigned int *)&MAC2[4],
		(unsigned int *)&MAC2[5]);


	sock = Inet_GetRawSock(iface, &MTU, &ifnumber, MyMAC);

	dest.sll_family  = AF_PACKET;
	dest.sll_ifindex = ifnumber;


	wait_for_signal = 0;

	for (;;)
	{
		char buffer[MTU];

		if (getppid() == 1) exit(0);

		{
			char ok2 = 0;
			memset(&data_to_ettercap, 0, sizeof(CONNECTION));

    		len = Inet_GetRawPacket(sock, buffer, MTU, &from);


			if (mode == ARPBased)
			{
				if (from.sll_pkttype == PACKET_HOST) ok2 = 1;
			}
			else
				ok2 = 1;


			if (len > 0 && ok2)
			{
				char ok = 0;
				char ok1 = 0;


				eth = (struct ETH_packet *)buffer;

				if (mode == MACBased)
				{
					if ( memcmp(eth->SourceMAC, MAC1, 6) == 0 || memcmp(eth->SourceMAC, MAC2, 6) == 0 ) ok = 1;
					if ( memcmp(eth->DestMAC, MAC1, 6) == 0 || memcmp(eth->DestMAC, MAC2, 6) == 0 ) ok = 1;
				}
				else
					ok = 1;

				if ( ok && ntohs(eth->Protocol_type) == ETHERTYPE_IP )
				{
					ip = (struct iphdr *)(eth+1);

					if (mode != MACBased)
					{
						if (ip->destIP==IP1 && ip->sourceIP==IP2)
						{
							if (mode == ARPBased)
							{
								memcpy(eth->DestMAC, MAC1, 6);
								memcpy(eth->SourceMAC, MyMAC, 6);
							}
							strcpy(data_to_ettercap.ip_source, IP2p);
							strcpy(data_to_ettercap.ip_dest, IP1p);
							ok1 = 1;
						}

						if (ip->destIP==IP2 && ip->sourceIP==IP1)
						{
							if (mode == ARPBased)
							{
								memcpy(eth->DestMAC, MAC2, 6);
								memcpy(eth->SourceMAC, MyMAC, 6);
							}
							strcpy(data_to_ettercap.ip_source, IP1p);
							strcpy(data_to_ettercap.ip_dest, IP2p);
							ok1 = 1;
						}
					}
					else
					{
						strcpy(data_to_ettercap.ip_source, inet_ntoa(*(struct in_addr *)&ip->sourceIP) );
						strcpy(data_to_ettercap.ip_dest, inet_ntoa(*(struct in_addr *)&ip->destIP) );
						ok1 = 1;
					}

					if ( ok1 && ip->proto == IPPROTO_TCP)
					{
						tcp = (struct tcphdr *)(ip + 1);

						data_to_ettercap.port_source = ntohs(tcp->source);
						data_to_ettercap.port_dest = ntohs(tcp->dest);

						if (wait_for_signal == 0) write(pipe_with_illithid[1], &data_to_ettercap, sizeof(CONNECTION));
					}

					if (mode == ARPBased && wait_for_signal == 0) Inet_SendRawPacket(sock, buffer, len, dest);
				}
			}
		}

	}

}



int Illithid_ToBeSniffed(char *source, int psource, char *dest, int pdest, SNIFFED_DATA data)
{

	char s=0, ps=0, d=0, pd=0;

	if (psource == 0) ps = 1;
	if (pdest == 0) pd = 1;
	if (!strcmp(source, "")) s = 1;
	if (!strcmp(dest, "")) d = 1;

	if (s || !strcmp(source, data.ip_source))
		if (ps || psource == data.port_source)
		{	s = 1;	ps = 1;	}

	if (s || !strcmp(source, data.ip_dest))
		if (ps || psource == data.port_dest)
		{	s = 1;	ps = 1;	}

	if (d || !strcmp(dest, data.ip_source))
		if (pd || pdest == data.port_source)
		{	d = 1;	pd = 1;	}

	if (d || !strcmp(dest, data.ip_dest))
		if (pd || pdest == data.port_dest)
		{	d = 1;	pd = 1;	}

	return ( s && ps && d && pd );

}

