/*
    ettercap -- inet utilities, arp ping and more...

    Copyright (C) 2001  ALoR <alor@thepentagon.com>, NaGA <crwm@freemail.it>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#include	<stdio.h>
#include	<stdlib.h>		//only for srand()
#include	<unistd.h>
#include	<string.h>
#include	<netdb.h>
#include	<arpa/inet.h>
#include	<sys/time.h>
#include	<netpacket/packet.h>
#include	<net/if_arp.h>
#include	<net/ethernet.h>
#include	<sys/ioctl.h>
#include	<net/if.h>
#include	<fcntl.h>
#include	<signal.h>
#include <errno.h>


#include	"ec_main.h"
#include "ec_error.h"

#ifdef DEBUG
	#include	"ec_debug.h"
#endif

typedef struct
{
	char DestMAC[6];
	char SourceMAC[6];
	unsigned	short	Protocol_type;
	unsigned	short	Hardware_type;
	unsigned	short	Res_protocol;
	char HA_len;
	char PA_len;
	unsigned	short	Op_code;
	char SourceAdd[6];
	char SourceIP[4];
	char DestAdd[6];
	char DestIP[4];
} ARP_packet;


typedef struct
{
	int IP_Add;
	unsigned	char MAC_Add[6];
	struct host_list *next;
} host_list;


struct ifreq old_ifr;		// old iface flags
char IpForward_status[2];	// old ipforward status

//	protos...

char * Inet_HostName(char *ip);
char * Inet_NameToIp(char *name);
char * Inet_GetMyInfo(char	tipo);
char * Inet_MyIPAddress(void);
char * Inet_MyMACAddress(void);
char * Inet_MySubnet(void);
char Inet_CorrectIface(char *iface);
int Inet_HostInLAN(void);
void Inet_Free_list(host_list	*head);
host_list *Inet_Host_in_LAN_list(char *interface);
int Inet_GetRawSock(char *iface, int *MTU, int *ifnumber, char *MyMAC);
int Inet_GetRawPacket(int sock, char *buffer, int MTU, struct sockaddr_ll *from);
int Inet_SendRawPacket(int sock, char *buffer, int len, struct sockaddr_ll dest);
unsigned short Inet_ChecksumIP(unsigned short *buffer, int size);
unsigned short Inet_ChecksumTCP(unsigned short *buffer, unsigned short size, unsigned long IPsource, unsigned long IPdest);
void Inet_Restore_ifr(void);
void Inet_DisableForwarding(void);
void Inet_RestoreForwarding(void);

//	----------------------------------------

char * Inet_HostName(char *ip)	//	returns hostname from ip
{
	struct hostent	*host;
	static struct in_addr addr;

	addr.s_addr	= inet_addr(ip);
	host = gethostbyaddr((char	*)&addr,	sizeof(struct in_addr),	AF_INET);

	if	(host	!=	NULL)	return (char *) host->h_name;

	return "Unknown host";
}


char * Inet_NameToIp(char *name)	//	returns ip from hostname
{
	struct hostent	*host;
	static char ip[16];

	host = gethostbyname(name);

	if	(host	!=	NULL)
		strncpy(ip, inet_ntoa( *(struct in_addr *) *host->h_addr_list), 16);
	else
		strncpy(ip, "(null)", 16);

	return ip;
}


char * Inet_GetMyInfo(char	tipo)
{
	struct ifreq ifr;
	struct in_addr	to_convert;
	int IP_Add,	sock,	NetMask;
	unsigned	char MAC_Add[6];
	static char	MAC[17];
	static char	MySubnet[50];
	static char	IP[16];

#ifdef DEBUG
	Debug_msg("Inet_GetMyInfo %d", tipo);
#endif

	sock = socket(PF_PACKET, SOCK_RAW, htons(ETH_P_ARP));
	if (sock < 0)
		Error_msg("ec_Inet:%d socket() | ERRNO : %d | %s \n", __LINE__, errno, sys_errlist[errno]);


	memset(&ifr, 0, sizeof(ifr));
	strncpy(ifr.ifr_name, Options.netiface, sizeof(ifr.ifr_name));

	switch (tipo)
	{
			case 0:	//	requesting IP
						if ( ioctl(sock, SIOCGIFADDR, &ifr) < 0)
							Error_msg("ec_Inet:%d ioctl(SIOCGIFADDR) | ERRNO : %d | %s \n", __LINE__, errno, sys_errlist[errno]);
						memcpy((char *)&(IP_Add), ifr.ifr_addr.sa_data + 2, 4);

						to_convert.s_addr	= IP_Add;
						sprintf(IP, "%s", inet_ntoa(to_convert));
						return IP;
						break;

			case 1:	//	requesting MAC
						if ( ioctl(sock, SIOCGIFHWADDR, &ifr) < 0 )
							Error_msg("ec_Inet:%d ioctl(SIOCGIFHWADDR) | ERRNO : %d | %s \n", __LINE__, errno, sys_errlist[errno]);
						memcpy(MAC_Add, ifr.ifr_hwaddr.sa_data, 6);

						sprintf(MAC,"%02X:%02X:%02X:%02X:%02X:%02X",
									MAC_Add[0],MAC_Add[1],MAC_Add[2],
									MAC_Add[3],MAC_Add[4],MAC_Add[5]);
						return MAC;
						break;

			case 2:	//	requesting netmask
						if ( ioctl(sock, SIOCGIFNETMASK, &ifr) < 0)
							Error_msg("ec_Inet:%d ioctl(SIOCGIFNETMASK) | ERRNO : %d | %s \n", __LINE__, errno, sys_errlist[errno]);
						memcpy((char *)&NetMask, ifr.ifr_addr.sa_data + 2, 4);

						to_convert.s_addr	= NetMask;
						sprintf(MySubnet, "%s", inet_ntoa(to_convert));
						return MySubnet;
						break;
	}

	return "(none)";

}



char * Inet_MyIPAddress(void)
{
	return Inet_GetMyInfo(0);
}



char * Inet_MyMACAddress(void)
{
	return Inet_GetMyInfo(1);
}



char * Inet_MySubnet(void)
{
	return Inet_GetMyInfo(2);
}




char Inet_CorrectIface(char *iface)
{
	int sock;
	struct sockaddr sa;
//	struct sockaddr_ll sll;
//	struct ifreq ifr;

#ifdef DEBUG
	Debug_msg("Inet_CorrectIface\t\tIface: %s", iface);
#endif

	sock = socket(PF_INET, SOCK_PACKET, htons(ETH_P_ALL));	// FIXME  deprecated !! should not be used...
//	sock = socket(PF_PACKET, SOCK_RAW, htons(ETH_P_ALL));		// but the new method doesn't work...
	if (sock < 0)
		Error_msg("ec_Inet:%d socket() | ERRNO : %d | %s", __LINE__, errno, sys_errlist[errno]);


//	memset(&ifr, 0, sizeof(ifr));
//	strncpy(ifr.ifr_name, iface, sizeof(ifr.ifr_name));
//	ioctl(sock, SIOCGIFINDEX, &ifr);	 // interafce number
//
//	memset(&sll, 0, sizeof(sll));
//	sll.sll_ifindex = ifr.ifr_ifindex;
//	sll.sll_family = AF_PACKET;
//	sll.sll_protocol = htons(ETH_P_ALL);

	memset(&sa, 0, sizeof(sa));
	sa.sa_family = AF_INET;
	strncpy(sa.sa_data, iface, sizeof(sa.sa_data));

//	if ( bind(sock, (struct sockaddr *) &sll, sizeof(sll)) == -1)
	if (bind(sock, (struct sockaddr *) &sa, sizeof(sa)))		// interface doesn't exist
		return -1;

	return 0;

}



int Inet_HostInLAN()
{
	host_list *list;
	host_list *index;
	int host_alive = 0, k = 0;

#ifdef DEBUG
	Debug_msg("Inet_HostInLAN\t\tIface: %s", Options.netiface);
#endif

	list = Inet_Host_in_LAN_list(Options.netiface);

	index = list;

	for( ; index; host_alive++)
		index = (host_list *)index->next;

	if (Host_In_LAN) free(Host_In_LAN);
	Host_In_LAN = (HOST *)calloc(host_alive,sizeof(HOST));
	if (Host_In_LAN == NULL)
		Error_msg("ec_Inet:%d calloc() | ERRNO : %d | %s", __LINE__, errno, sys_errlist[errno]);

	index = list;
	for( ; index; index = (host_list *)index->next)
	{
		struct in_addr to_convert;
		to_convert.s_addr = index->IP_Add;
		sprintf(Host_In_LAN[k].ip,"%s",inet_ntoa(to_convert));
		snprintf(Host_In_LAN[k].name, 128, "%s", Inet_HostName(Host_In_LAN[k].ip));
		sprintf(Host_In_LAN[k].mac,"%02X:%02X:%02X:%02X:%02X:%02X",
			index->MAC_Add[0],index->MAC_Add[1],index->MAC_Add[2],
			index->MAC_Add[3],index->MAC_Add[4],index->MAC_Add[5]);
		k++;
	}
	Inet_Free_list(list);

	return host_alive;
}




void Inet_Free_list(host_list *head)
{
	if (!head) return;

	Inet_Free_list((host_list *) head->next);
	free(head);
}



host_list *Inet_Host_in_LAN_list(char *interface)
{
	int ifnumber, sock, NetMask, N_hosts, index, to_end;
	struct sockaddr_ll dest;
	struct ifreq ifr;
	host_list *head;
	ARP_packet pkarp;

	void handle(int dummy)
	{
		to_end=0;
		return;
	}

	head = (host_list *)malloc(sizeof(host_list));
	if (head == NULL)
			Error_msg("ec_Inet:%d malloc() | ERRNO : %d | %s", __LINE__, errno, sys_errlist[errno]);
	head->next=NULL;

	sock = socket(PF_PACKET, SOCK_RAW, htons(ETH_P_ARP));
	if (sock < 0)
		Error_msg("ec_Inet:%d socket() | ERRNO : %d | %s", __LINE__, errno, sys_errlist[errno]);

	memset(&ifr, 0, sizeof(ifr));
	strncpy(ifr.ifr_name, interface, sizeof(ifr.ifr_name));

	if ( ioctl(sock, SIOCGIFINDEX, &ifr) < 0 )		// interafce number
		Error_msg("ec_Inet:%d ioctl(SIOCGIFINDEX) | ERRNO : %d | %s \n", __LINE__, errno, sys_errlist[errno]);
	ifnumber = ifr.ifr_ifindex;
	if ( ioctl(sock, SIOCGIFHWADDR, &ifr) < 0 )		// MAC
		Error_msg("ec_Inet:%d ioctl(SIOCGIFHWADDR) | ERRNO : %d | %s \n", __LINE__, errno, sys_errlist[errno]);
	memcpy(head->MAC_Add, ifr.ifr_hwaddr.sa_data, 6);
	if ( ioctl(sock, SIOCGIFADDR, &ifr) < 0 )			// IP
		Error_msg("ec_Inet:%d ioctl(SIOCGIFADDR) | ERRNO : %d | %s \n", __LINE__, errno, sys_errlist[errno]);
	memcpy((char *)&(head->IP_Add), ifr.ifr_addr.sa_data+2, 4);
	if ( ioctl(sock, SIOCGIFNETMASK, &ifr) < 0 )		// NetMask
		Error_msg("ec_Inet:%d ioctl(SIOCGIFNETMASK) | ERRNO : %d | %s \n", __LINE__, errno, sys_errlist[errno]);
	memcpy((char *)&NetMask, ifr.ifr_addr.sa_data+2, 4);

	N_hosts = ntohl(~NetMask);

	dest.sll_family = AF_PACKET;
	dest.sll_ifindex = ifnumber;

	//	frame ethernet header
	pkarp.Protocol_type = htons(ETHERTYPE_ARP);
	pkarp.Hardware_type = htons(ARPHRD_ETHER);
	pkarp.Res_protocol = htons(ETHERTYPE_IP);
	pkarp.HA_len = 6;
	pkarp.PA_len = 4;
	pkarp.Op_code = htons(ARPOP_REQUEST);
	memcpy(pkarp.SourceMAC, head->MAC_Add, 6);
	memcpy(pkarp.SourceAdd, head->MAC_Add, 6);
	memcpy(pkarp.SourceIP, (char *)&(head->IP_Add), 4);
	memset(pkarp.DestMAC,0xFF,6);
	memset(pkarp.DestAdd,0,6);

	for (index=1; index<=N_hosts; index++)
	{
		int DestIP;
		DestIP = (head->IP_Add&NetMask)|htonl(index);

		// if dest is equal to me
		if (DestIP != head->IP_Add)
		{
			memcpy(pkarp.DestIP, (char *)&DestIP, 4);
			sendto( sock, (char *)&pkarp, sizeof(ARP_packet), 0,(struct sockaddr *)&dest, sizeof(dest) );
		}
		usleep(500);
	}

		to_end = 1;
		signal(SIGALRM, handle);
		alarm(2);
		fcntl(sock, F_SETFL, O_NONBLOCK);

	//	wait for reply
	do
	{
		int fromlen,leng;
		struct sockaddr_ll from;
		host_list **index;
		ARP_packet recvpk;

		fromlen = sizeof(from);
		leng = recvfrom(sock, &recvpk, sizeof(ARP_packet), 0, (struct sockaddr *)&from, &fromlen);

		if (leng > 0 && from.sll_pkttype == PACKET_HOST && recvpk.Op_code == htons(ARPOP_REPLY))
		{
			index = &head;

			while(*index != NULL && memcmp(&((*index)->IP_Add),recvpk.SourceIP,4))
				index=(host_list **)&((*index)->next);

			if (*index == NULL)
			{
				*index=(host_list *)malloc(sizeof(host_list));
				(*index)->next=NULL;
				memcpy((*index)->MAC_Add, recvpk.SourceAdd, 6);
				memcpy((char *)&((*index)->IP_Add), recvpk.SourceIP, 4);
			}
		}
	} while (to_end);

	return head;
}



int Inet_GetRawSock(char *iface, int *MTU, int *ifnumber, char *MyMAC)
{
	int sock;
	struct sockaddr_ll sll;
	struct ifreq ifr;

#ifdef DEBUG
    Debug_msg("Inet_GetRawSock %s", iface);
#endif

	sock = socket(PF_PACKET, SOCK_RAW, htons(ETH_P_ALL));
	if (sock < 0)
		Error_msg("ec_Inet:%d socket() | ERRNO : %d | %s", __LINE__, errno, sys_errlist[errno]);


	memset(&ifr, 0, sizeof(ifr));
	strncpy(ifr.ifr_name, iface, sizeof(ifr.ifr_name));
	if ( ioctl(sock, SIOCGIFMTU, &ifr) < 0)
	{
		#ifdef DEBUG
			Debug_msg("Inet_IPBased_Run -- MTU FAILED... assuming 2000");
		#endif
		*MTU = 2000;
	}
	else
		*MTU = ifr.ifr_mtu + 256;

	memset(&ifr, 0, sizeof(ifr));
	strncpy(ifr.ifr_name, iface, sizeof(ifr.ifr_name));

	if ( ioctl(sock, SIOCGIFINDEX, &ifr) < 0)
		Error_msg("ec_Inet:%d ioctl(SIOCGIFINDEX) | ERRNO : %d | %s \n", __LINE__, errno, sys_errlist[errno]);
	if (ifnumber != NULL) *ifnumber = ifr.ifr_ifindex;

	memset(&sll, 0, sizeof(sll));
	sll.sll_family = AF_PACKET;
	sll.sll_ifindex = ifr.ifr_ifindex;
	sll.sll_protocol = htons(ETH_P_ALL);

	if (bind(sock, (struct sockaddr *) &sll, sizeof(sll)) == -1)
		Error_msg("ec_Inet:%d bind() | ERRNO : %d | %s", __LINE__, errno, sys_errlist[errno]);


	if ( ioctl(sock, SIOCGIFHWADDR, &ifr) < 0 )
		Error_msg("ec_Inet:%d ioctl(SIOCGIFHWADDR) | ERRNO : %d | %s \n", __LINE__, errno, sys_errlist[errno]);
	if (MyMAC != NULL) memcpy(MyMAC, ifr.ifr_hwaddr.sa_data, 6);


	if ( ioctl(sock, SIOCGIFFLAGS, &ifr) < 0 )
		Error_msg("ec_Inet:%d ioctl(SIOCGIFFLAGS) | ERRNO : %d | %s \n", __LINE__, errno, sys_errlist[errno]);
	old_ifr = ifr;		//save old flags
	ifr.ifr_flags |= IFF_PROMISC;
	if ( ioctl(sock, SIOCSIFFLAGS, &ifr) < 0 )		// promisc mode
		Error_msg("ec_Inet:%d ioctl(SIOCSIFFLAGS) | promisc on | ERRNO : %d | %s \n", __LINE__, errno, sys_errlist[errno]);

	exit_func(Inet_Restore_ifr);

	return sock;
}



void Inet_Restore_ifr(void)
{
	int sock;

#ifdef DEBUG
    Debug_msg("Inet_Restore_ifr");
#endif

   sock = socket(PF_PACKET, SOCK_RAW, htons(ETH_P_ALL));
	if ( ioctl(sock, SIOCSIFFLAGS, &old_ifr) < 0 )		// flag restoring
		Error_msg("ec_Inet:%d ioctl(SIOCSIFFLAGS) | flag restoring | ERRNO : %d | %s \n", __LINE__, errno, sys_errlist[errno]);


}



int Inet_GetRawPacket(int sock, char *buffer, int MTU, struct sockaddr_ll *from)
{
	int len = 0, fromlen;

	fromlen = sizeof(struct sockaddr_ll);
	len = recvfrom(sock, buffer, MTU, 0, (struct sockaddr *)from, &fromlen);

	// TODO
	// handling for fragmented packets...

	return len;
}



int Inet_SendRawPacket(int sock, char *buffer, int len, struct sockaddr_ll dest)
{

	return sendto(sock, buffer, len, 0, (struct sockaddr *)&dest, sizeof(dest));

}



unsigned short Inet_ChecksumIP(unsigned short *buffer, int size)
{
	unsigned long cksum=0;

	while(size >1)
	{
		cksum += *buffer++;
		size -= sizeof(unsigned short);
	}

	if(size ) cksum += *(unsigned char*)buffer;

	cksum = (cksum >> 16) + (cksum & 0xffff);
	cksum += (cksum >>16);
	return (unsigned short)(~cksum);

}



unsigned short Inet_ChecksumTCP(unsigned short *buffer, unsigned short size, unsigned long IPsource, unsigned long IPdest)
{
	unsigned long cksum=0;
	unsigned short proto;
	unsigned short len;

	proto = htons(IPPROTO_TCP);
	len = size;

	while(size >1)
	{
		cksum+=*buffer++;
		size -=sizeof(unsigned short);
	}

	cksum += IPsource >> 16;
	cksum += IPsource & 0xffff;
	cksum += IPdest >> 16;
	cksum += IPdest & 0xffff;
	cksum += proto;
	cksum += htons(len);

	if(size ) cksum += *(unsigned char*)buffer;

	cksum = (cksum >> 16) + (cksum & 0xffff);
	cksum += (cksum >>16);
	return (unsigned short)(~cksum);

}


void Inet_DisableForwarding(void)
{
	FILE *fd;

	fd = fopen("/proc/sys/net/ipv4/ip_forward", "r");
	if (fd < 0 )
	{
		#ifdef DEBUG
			Debug_msg("ec_Inet:%d fopen(/proc/sys/net/ipv4/ip_forward) | ERRNO : %d | %s \n", __LINE__, errno, sys_errlist[errno]);
		#endif
		return;
	}

	fscanf(fd, "%s", IpForward_status);
	fclose(fd);

#ifdef DEBUG
    Debug_msg("Inet_DisableForwarding from %c", IpForward_status[0]);
#endif

	exit_func(Inet_RestoreForwarding);

	fd = fopen("/proc/sys/net/ipv4/ip_forward", "w");
	if (fd < 0 )
	{
		#ifdef DEBUG
			Debug_msg("ec_Inet:%d fopen(/proc/sys/net/ipv4/ip_forward) | ERRNO : %d | %s \n", __LINE__, errno, sys_errlist[errno]);
		#endif
		return;
	}

	fprintf(fd, "0");
	fclose(fd);

}



void Inet_RestoreForwarding(void)
{
	FILE *fd;

	fd = fopen("/proc/sys/net/ipv4/ip_forward", "w");
	if (fd < 0 )
	{
		#ifdef DEBUG
			Debug_msg("ec_Inet:%d fopen(/proc/sys/net/ipv4/ip_forward) | ERRNO : %d | %s \n", __LINE__, errno, sys_errlist[errno]);
		#endif
		return;
	}

#ifdef DEBUG
    Debug_msg("Inet_RestoreForwarding to %c", IpForward_status[0]);
#endif

	fprintf(fd, "%c", IpForward_status[0] );
	fclose(fd);
}

