/*
    ettercap -- module for NON ncurses interface

    Copyright (C) 2001  ALoR <alor@thepentagon.com>, NaGA <crwm@freemail.it>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <stdlib.h>
#include <fcntl.h>
#include <sys/ioctl.h>
#include <sys/time.h>
#include <signal.h>

#include "include/ec_main.h"
#include "include/ec_error.h"
#include "include/ec_inet.h"
#include "include/ec_decodedata.h"
#include "include/ec_illithid.h"
#include "include/ec_doppleganger.h"
#include "include/ec_buffer.h"
#include "include/ec_plugins.h"

#ifdef DEBUG
	#include "include/ec_debug.h"
#endif

#ifdef HAVE_TERMIOS_H

	#include <termios.h>

	struct termios old_tc;
	struct termios new_tc;

	void set_raw_term(void);
	void reset_term(void);

#endif

#define ASCII_VIEW   0
#define HEX_VIEW		1

extern char *Execute_Plugin;

// protos...
#ifdef PERMIT_PLUGINS
	void Simple_Plugin(void);
#endif
void Simple_HostList(void);
void Simple_Run(void);
void Simple_CheckForPoisoner(void);

//---------------------------------

void Simple_HostList(void)
{
	int j;

#ifdef DEBUG
	Debug_msg("Simple_HostList");
#endif

	printf("Host in this LAN:\n\n");

	for(j=0; j<number_of_hosts_in_lan; j++)
	{
		printf("%3d) %s\n", j+1, Host_In_LAN[j].ip);
	}

	printf("\n");
}


void Simple_CheckForPoisoner(void)
{
	SniffingHost *SniffList;
	int i;
	short found = 0;


#ifdef DEBUG
	Debug_msg("Simple_CheckForPoisoner");
#endif


	printf("Checking for poisoners...\n\n");

	SniffList = Inet_NoSniff();

	for (i=0; i<number_of_hosts_in_lan; i++)
	{
		if (SniffList[i].mode == 0) break;
		if (SniffList[i].mode == 1)
		{
			found = 1;
			printf("%s is replying for %s", Host_In_LAN[SniffList[i].Host_Index1].ip, Host_In_LAN[SniffList[i].Host_Index2].ip);
		}

		if (SniffList[i].mode == 2)
		{
			found = 1;
			printf("MAC of %s and %s are identical !",Host_In_LAN[SniffList[i].Host_Index1].ip,Host_In_LAN[SniffList[i].Host_Index2].ip);
		}

	}

	if (!found)
		printf("No poisoners found in this lan (seems to be safe)\n\n");


}

#ifdef PERMIT_PLUGINS

void Simple_Plugin(void)
{
	char plugin[strlen(Execute_Plugin)+6];
	short i, n, found=-1;
	#include "include/ec_plugins_path.h"

#ifdef DEBUG
	Debug_msg("Simple_Plugin -- %s", Execute_Plugin);
#endif


	if (strcmp(Host_Source.name, ""))
		strcpy(Host_Source.ip, Inet_NameToIp(Host_Source.name));

	if (strcmp(Host_Dest.name, ""))
		strcpy(Host_Dest.ip, Inet_NameToIp(Host_Dest.name));

#ifdef DEBUG
	Debug_msg("Simple_Plugin -- [%s] [%s] [%s] [%s]", Host_Source.name, Host_Source.ip, Host_Dest.name, Host_Dest.ip );
#endif

	strcpy(plugin, "ec_");
	strcat(plugin, Execute_Plugin);
	strcat(plugin, ".so");

	n = Plugin_MakePlugList();

	if (!strcmp(Execute_Plugin, "list"))
	{
		fprintf(stdout, "\n Available Plugins :\n\n");

		for(i=0; i<n; i++)
			fprintf(stdout, "%2d) %s \n", i+1, Plugin_Getname(Plugins_list[i].name));
		return;
	}

	for(i=0; i<n; i++)
	{
		if ( !strcmp(Plugins_list[i].name, plugin) ) found = i;
	}

	if (found == -1)
	{
		fprintf(stderr, "\nPlugin %s NOT found in %s or in ./ !\n", plugin, path);
		fprintf(stderr, "\nuse -p list to view the available ones.\n\n");
	}
	else
	{
		char toexec[strlen(plugin)+strlen(Plugins_list[found].path)];
		strcpy(toexec, Plugins_list[found].path);
		strcat(toexec, plugin);
		Plugin_RunPlugIn(toexec, NULL);
	}

}

#endif


#ifdef HAVE_TERMIOS_H

void set_raw_term()			// taken from readchar.c, by M. Andreoli (2000)
{
    tcgetattr(0, &old_tc);
    new_tc = old_tc;
    new_tc.c_lflag &= ~(ECHO | ICANON);	/* raw output */
    new_tc.c_cc[VTIME] = 1;

    tcsetattr(0, TCSANOW, &new_tc);
}

void reset_term()				// taken from readchar.c, by M. Andreoli (2000)
{
    tcsetattr(0, TCSANOW, &old_tc);
}

#endif



void Simple_Run(void)
{
	int Illithid_pid = 0, Dopple_pid = 0;
	int i, index_source=-1, index_dest=-1;
	char view = ASCII_VIEW;
	char stop = 0;
	fd_set msk_fd;
	struct timeval TimeOut;
	SNIFFED_DATA data_from_illithid;
	int datalen;


#ifdef DEBUG
	Debug_msg("Simple_Run -- %s <--> %s", Host_Source.name, Host_Dest.name);
#endif

	if (strcmp(Host_Source.name, ""))
		strcpy(Host_Source.ip, Inet_NameToIp(Host_Source.name));
	else
		strcpy(Host_Source.ip, "");

	if (strcmp(Host_Dest.name, ""))
		strcpy(Host_Dest.ip, Inet_NameToIp(Host_Dest.name));
	else
		strcpy(Host_Dest.ip, "");

	for(i=0; i<number_of_hosts_in_lan; i++)
	{
		if ( !strcmp(Host_Source.ip, Host_In_LAN[i].ip) ) index_source = i;
		if ( !strcmp(Host_Dest.ip, Host_In_LAN[i].ip) ) index_dest = i;
	}


	if ( index_source < 0 && strcmp(Host_Source.name, ""))
		Error_msg("Source host %s (%s) not found !!", Host_Source.name, Host_Source.ip);


	if ( index_dest < 0 && strcmp(Host_Dest.name, ""))
		Error_msg("Dest host %s (%s) not found !!", Host_Dest.name, Host_Dest.ip);

	if (!strcmp(Host_Source.ip, Host_Dest.ip) && strcmp(Host_Source.name, ""))
		Error_msg("Source and Dest MUST be different !!");

	if (Options.arpsniff)
		if ( (!strcmp(Host_Source.ip, Inet_MyIPAddress())) || (!strcmp(Host_Dest.ip, Inet_MyIPAddress())) )
			Error_msg("CAN'T arpsniff yourself !!");

	// it is ok...

	if (Options.arpsniff)		// Doppleganger born...
		Dopple_pid = Doppleganger_Run(Options.netiface, Host_In_LAN[index_source].ip, Host_In_LAN[index_dest].ip, Host_In_LAN[index_source].mac, Host_In_LAN[index_dest].mac);

	printf("\nPress 'h' for help...\n\n");

	pipe_with_illithid_data = Buffer_Create(1.0e6); // 1Mb in shared memory !!!

	if (Options.arpsniff)
	{
		printf(" Sniffing (ARP based) : %s:%d <--> %s <--> %s:%d\n\n", Host_Source.ip, Host_Source.port, Inet_MyIPAddress(), Host_Dest.ip, Host_Dest.port);
		Illithid_pid = Illithid_ARPBased_GetData(Options.netiface, Host_In_LAN[index_source].ip, Host_Source.port, Host_In_LAN[index_dest].ip, Host_Dest.port, Host_In_LAN[index_source].mac, Host_In_LAN[index_dest].mac);
	}
	else if (Options.sniff)
	{
		printf(" Sniffing (IP based): %s:%d <--> %s:%d\n\n", Host_Source.ip, Host_Source.port, Host_Dest.ip, Host_Dest.port );
		Illithid_pid = Illithid_IPBased_GetData(Options.netiface, Host_Source.ip, Host_Source.port, Host_Dest.ip, Host_Dest.port);
	}
	else if (Options.macsniff)
	{
		printf(" Sniffing (MAC based): %s <--> %s\n\n", Host_Source.ip, Host_Dest.ip);
		Illithid_pid = Illithid_MACBased_GetData(Options.netiface, Host_In_LAN[index_source].mac, Host_In_LAN[index_dest].mac);
	}



		memset(&TimeOut, 0, sizeof(TimeOut));	//  timeout = 0
		FD_ZERO(&msk_fd);


#ifdef HAVE_TERMIOS_H
		set_raw_term();	// non blocking stdin... yes this work !!
#else
	#ifdef DEBUG
		Debug_msg("Simple_Run -- NO TERMIOS_H");
	#endif
		fcntl(0, F_SETFL, O_NONBLOCK);	// stdin non blocking... seems to be a non working method for me...
#endif

		loop
		{

			FD_SET(0, &msk_fd);

			select(FOPEN_MAX, &msk_fd, (fd_set *) 0, (fd_set *) 0, &TimeOut);

			if (FD_ISSET(0, &msk_fd))
			{
				char ch = 0;
				ch = getchar();
				switch(ch)
				{
					case 'A':
					case 'a':
							printf("\n\nAscii mode...\n\n");
							view = ASCII_VIEW;
							break;

					case 'X':
					case 'x':
							printf("\n\nHex mode...\n\n");
							view = HEX_VIEW;
							break;

					case 'S':
					case 's':
							if (stop == 1)
							{
								printf("\n\nLet's go...\n\n");
								stop = 0;
							}
							else
							{
								printf("\n\nStopped...\n\n");
								stop = 1;
							}
							break;

					case 'H':
					case 'h':
							printf("\n\n[qQ]  - quit\n");
							printf(    "[aA]  - dump data in ascii mode\n");
							printf(    "[xX]  - dump data in hex mode\n");
							printf(    "[sS]  - stop/cont sniffing\n\n");
							break;

					case 'Q':
					case 'q':
#ifdef HAVE_TERMIOS_H
							reset_term();
#endif
							printf("\n\n\nQuitting...\n\n");
							kill(Illithid_pid, SIGTERM);
							if (Dopple_pid) kill(Dopple_pid, SIGTERM);
							exit(0);
							break;
				}
			}

			datalen = Buffer_Get(pipe_with_illithid_data, &data_from_illithid, sizeof(SNIFFED_DATA));

			if (datalen>0)
			{
					if (!stop && data_from_illithid.datasize != 0)
						switch (view)
						{
							case ASCII_VIEW:
								printf("\n\n%s:%d -> %s:%d\n%s",
									data_from_illithid.ip_source,
									data_from_illithid.port_source,
									data_from_illithid.ip_dest,
									data_from_illithid.port_dest,
									Decodedata_GetAsciiData(data_from_illithid.data, data_from_illithid.datasize)
								);
								break;

							case HEX_VIEW:
								printf("\n\n%s:%d -> %s:%d\n%s",
									data_from_illithid.ip_source,
									data_from_illithid.port_source,
									data_from_illithid.ip_dest,
									data_from_illithid.port_dest,
									Decodedata_GetHexData(data_from_illithid.data, data_from_illithid.datasize, 80)
								);
								break;
						}
					fflush(stdout);
			}
			else
			    usleep(1000);
		}
}

/* EOF */
