/*

  This file is part of sniffer, a packet capture utility and
  network moniter
  The author can be contacted at <mistral@stev.org>
  the lastest version is avilable from 
  http://stev.org

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#ifndef _SNIFF_FTP_H
#define _SNIFF_FTP_H

#include "config.h"
#include "list.h"
#include "tcp.h"


#ifndef _SNIFF_TCP_H
#include "tcp.h"
#endif

#ifdef FTP

/* ftp status numbers */
#define FTP_ERROR 0   /* we use this to say we did not get any good info */
#define FTP_START 1   /* we have just spotted a connection */
#define FTP_OK    3   /* after the server gives 220 */
#define FTP_PASS  4   /* the user is about to send the password */
#define FTP_LOGIN 5   /* the user has logged in :) we should have the password */
#define FTP_BYE   255  /* the server is saying goodbye */

struct ftp_data {
	unsigned char state;  /* what state the ftp is in */

	char *user;
	char *pass;
	char *acct;
	char *syst;
	char *pwd;
	struct list_t *welcome; /* used to store the welcome message */
	struct list_t *download;
	struct list_t *upload;

};

extern pthread_mutex_t ftp_mutex;

extern void ftp_src			(struct sniff_pkt *pkt, struct tcp_data *tcp , char *data , int length );
extern void ftp_dst			(struct sniff_pkt *pkt, struct tcp_data *tcp , char *data , int length );
extern void ftp_clean		(struct tcp_data *tcp );
extern void ftp_init		(struct tcp_data *tcp );
extern void ftp_open		( void );

#endif /* ifdef FTP */

#endif /* _SNIFF_FTP_H */

